Block

Struct Block 

pub struct Block<T> {
    len: AtomicUsize,
    data: AtomicPtr<WriteOnce<T>>,
}
An allocation that can store a fixed number of elements.

Fields

len: AtomicUsize

The number of elements in the block.

data: AtomicPtr<WriteOnce<T>>

A pointer to the data buffer.

If size is nonzero, this points to a slice of WriteOnce<T> of that size. If size is zero, this is a null pointer.

It would be more convenient to use AtomicPtr<[WriteOnce<T>] here. However, AtomicPtr<T> requires T: Sized. This is because pointers to DSTs are fat, and take two words in memory. Most architectures can operate atomically on at most one word.

Implementations

Source

impl<T> Block<T>

pub(super) fn new() -> Self

Create a new block.

pub(super) fn is_init(&self) -> bool

pub(super) fn data(&self) -> &AtomicPtr<WriteOnce<T>>

pub(super) unsafe fn init(&self, size: NonZero<usize>)

Initialize the block with a given size.

Safety

For any given block, this method must always be called with the same size.

pub(super) unsafe fn insert(&self, index: usize, value: T)

Insert a value at the given index.

Safety

The block must be initialized.

pub(super) unsafe fn try_insert(&self, index: usize, value: T) -> Result<(), T>

Attempt to insert a value at the given index.

Safety

The block must be initialized.

pub(super) unsafe fn get(&self, index: usize) -> Option<&T>

Return the value at the given index.

Safety

The block must be initialized.

pub(super) fn get_mut(&mut self, index: usize) -> Option<&mut T>

Return a mutable reference to the value at the given index if it exists.

This method is safe to call even if the block is uninitialized.

pub(super) unsafe fn is_set(&self, index: usize) -> bool

Return the value at the given index.

Safety

The block must be initialized.