struct IterFrame<R> {
depth: usize,
current_node: R,
range: Range<i32>,
}Expand description
A stack frame in the depth-first traversal of a KdTrie.
Each frame records the current node, its depth in the trie (i.e. which coordinate dimension it indexes), and the remaining range of indices to visit at this node.
Fields
depth: usizecurrent_node: Rrange: Range<i32>Auto Trait Implementations
impl<R> Freeze for IterFrame<R>where
R: Freeze,
impl<R> RefUnwindSafe for IterFrame<R>where
R: RefUnwindSafe,
impl<R> Send for IterFrame<R>where
R: Send,
impl<R> Sync for IterFrame<R>where
R: Sync,
impl<R> Unpin for IterFrame<R>where
R: Unpin,
impl<R> UnwindSafe for IterFrame<R>where
R: UnwindSafe,