OooTracker

Struct OooTracker 

pub struct OooTracker(BTreeSet<usize>);
A wrapper around a BTreeSet that tracks out-of-order element insertions.

This is an internal implementation detail of OnceVec that keeps track of indices where elements have been inserted out of order. It’s also used as the target of the mutex lock to prevent concurrent modifications.

See OnceVec documentation for more details on how out-of-order insertions work.

Tuple Fields

0: BTreeSet<usize>