pub fn _mm256_castph256_ph128(a: __m256h) -> __m128h
Cast vector of type __m256h to type __m128h. This intrinsic is only used for compilation and does not generate any instructions, thus it has zero latency.
__m256h
__m128h
Intel’s documentation