pub fn _mm256_castph_pd(a: __m256h) -> __m256d
Cast vector of type __m256h to type __m256d. This intrinsic is only used for compilation and does not generate any instructions, thus it has zero latency.
__m256h
__m256d
Intel’s documentation