pub fn _mm256_castph_si256(a: __m256h) -> __m256i
Cast vector of type __m256h to type __m256i. This intrinsic is only used for compilation and does not generate any instructions, thus it has zero latency.
__m256h
__m256i
Intel’s documentation