pub fn _mm256_castsi256_ph(a: __m256i) -> __m256h
Cast vector of type __m256i to type __m256h. This intrinsic is only used for compilation and does not generate any instructions, thus it has zero latency.
__m256i
__m256h
Intel’s documentation