pub fn _mm256_fnmadd_ph(a: __m256h, b: __m256h, c: __m256h) -> __m256hExpand description
Multiply packed half-precision (16-bit) floating-point elements in a and b, subtract the intermediate result from packed elements in c, and store the results in dst.