pub fn _mm512_castph512_ph256(a: __m512h) -> __m256h
Cast vector of type __m512h to type __m256h. This intrinsic is only used for compilation and does not generate any instructions, thus it has zero latency.
__m512h
__m256h
Intel’s documentation