pub enum UnwindTerminateReason {
Abi,
InCleanup,
}🔬This is a nightly-only experimental API. (
custom_mir)Expand description
The reason we are terminating the process during unwinding.
Variants
Abi
🔬This is a nightly-only experimental API. (
custom_mir)Unwinding is just not possible given the ABI of this function.
InCleanup
🔬This is a nightly-only experimental API. (
custom_mir)We were already cleaning up for an ongoing unwind, and a second, nested unwind was triggered by the drop glue.