pub struct Block<T> {
len: AtomicUsize,
data: AtomicPtr<WriteOnce<T>>,
}Expand description
An allocation that can store a fixed number of elements.
Fields
len: AtomicUsizeThe number of elements in the block.
data: AtomicPtr<WriteOnce<T>>A pointer to the data buffer.
If size is nonzero, this points to a slice of WriteOnce<T> of that size. If size is
zero, this is a null pointer.
It would be more convenient to use AtomicPtr<[WriteOnce<T>] here. However, AtomicPtr<T>
requires T: Sized. This is because pointers to DSTs are fat, and take two words in memory.
Most architectures can operate atomically on at most one word.
Implementations
Sourceimpl<T> Block<T>
impl<T> Block<T>
pub(super) fn is_init(&self) -> bool
pub(super) fn data(&self) -> &AtomicPtr<WriteOnce<T>>
Sourcepub(super) unsafe fn init(&self, size: NonZero<usize>)
pub(super) unsafe fn init(&self, size: NonZero<usize>)
Initialize the block with a given size.
Safety
For any given block, this method must always be called with the same size.
Sourcepub(super) unsafe fn try_insert(&self, index: usize, value: T) -> Result<(), T>
pub(super) unsafe fn try_insert(&self, index: usize, value: T) -> Result<(), T>