pub struct OooTracker(BTreeSet<usize>);Expand description
A wrapper around a BTreeSet that tracks out-of-order element insertions.
This is an internal implementation detail of OnceVec that keeps track of
indices where elements have been inserted out of order. It’s also used as the
target of the mutex lock to prevent concurrent modifications.
See OnceVec documentation for more details on how out-of-order insertions work.
Tuple Fields
0: BTreeSet<usize>Trait Implementations
Sourceimpl Clone for OooTracker
impl Clone for OooTracker
Sourcefn clone(&self) -> OooTracker
fn clone(&self) -> OooTracker
Returns a duplicate of the value. Read more
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreSourceimpl Default for OooTracker
impl Default for OooTracker
Sourcefn default() -> OooTracker
fn default() -> OooTracker
Returns the “default value” for a type. Read more