#[repr(u8)]enum WriteOnceState {
Uninit = 0,
Writing = 1,
Init = 2,
}Expand description
The possible states of a WriteOnce.
We distinguish between Uninit and Writing so that we reach the Err branch of set if
set has been called by any thread before.
We distinguish between Writing and Init so that loading Init has a happens-before
relationship with the write in set.
Variants